package com.aurellem.capture.examples;

import java.io.File;

import com.aurellem.capture.IsoTimer;
import com.aurellem.capture.audio.MultiListener;
import com.aurellem.capture.audio.WaveFileWriter;
import com.jme3.app.SimpleApplication;
import com.jme3.audio.AudioNode;
import com.jme3.audio.AudioParam;
import com.jme3.audio.Listener;
import com.jme3.cinematic.MotionPath;
import com.jme3.cinematic.events.MotionTrack;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.MouseButtonTrigger;
import com.jme3.light.DirectionalLight;
import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.shape.Box;
import com.jme3.scene.shape.Sphere;
import com.jme3.system.AppSettings;


/**
 * 
 * Demonstrates advanced use of the audio capture and recording features.
 * Multiple perspectives of the same scene are simultaneously rendered to 
 * different sound files.  
 * 
 * A key limitation of the way multiple listeners are implemented is that 
 * only 3D positioning effects are realized for listeners other than the
 * main LWJGL listener.  This means that audio effects such as environment
 * settings will *not* be heard on any auxiliary listeners, though sound 
 * attenuation will work correctly.  
 * 
 * Multiple listeners as realized here might be used to make AI entities 
 * that can each hear the world from their own perspective.  
 * 
 * @author Robert McIntyre
 *
 */

public class AdvancedAudio extends SimpleApplication {

	  public static void main(String[] args) {
		  
			AdvancedAudio app = new AdvancedAudio();
			AppSettings settings = new AppSettings(true);
			//settings.setAudioRenderer("Send");
			app.setSettings(settings);
			app.setShowSettings(false);
			app.setPauseOnLostFocus(false);
			org.lwjgl.input.Mouse.setGrabbed(false);
			app.start();
	  }
	 
	 private MotionTrack motionControl;
	
	 
	 private void makeEar(Node root, Vector3f position){
		 Material mat = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
		 Geometry ear = new Geometry("ear", new Box(1.0f, 1.0f, 1.0f));
		 ear.setLocalTranslation(position);
		 mat.setColor("Color", ColorRGBA.Green);
		 ear.setMaterial(mat);
		 root.attachChild(ear);
	 } 
	 
	 private Geometry bell;
	 
	 private void createScene() {
		 Material mat = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
		 bell = new Geometry( "sound-emitter" , new Sphere(15,15,1));
		 mat.setColor("Color", ColorRGBA.Blue);
		 bell.setMaterial(mat);
		 rootNode.attachChild(bell);
		 
		 DirectionalLight light = new DirectionalLight();
		 light.setDirection(new Vector3f(0, -1, 0).normalizeLocal());
		 light.setColor(ColorRGBA.White.mult(1.5f));
		 rootNode.addLight(light);

		 makeEar(rootNode, new Vector3f(0, 0 ,20));
		 makeEar(rootNode, new Vector3f(0, 0 ,-20));
		 makeEar(rootNode, new Vector3f(20, 0 ,0));
		 makeEar(rootNode, new Vector3f(-20, 0 ,0));

		 MotionPath path = new MotionPath();
		 
		 // loop 1
		 path.addWayPoint(new Vector3f(0, 0, 0));
		 path.addWayPoint(new Vector3f(0, 0, -10));
		 path.addWayPoint(new Vector3f(-2, 0, -14));
		 path.addWayPoint(new Vector3f(-6, 0, -20));
		 path.addWayPoint(new Vector3f(0, 0, -26));
		 path.addWayPoint(new Vector3f(6, 0, -20));
		 path.addWayPoint(new Vector3f(0, 0, -14));
		 path.addWayPoint(new Vector3f(-6, 0, -20));
		 path.addWayPoint(new Vector3f(0, 0, -26));
		 path.addWayPoint(new Vector3f(6, 0, -20));
		 	 
		 
		 // loop 2
		 path.addWayPoint(new Vector3f(5, 0, -5));
		 path.addWayPoint(new Vector3f(7, 0, 1.5f));
		 path.addWayPoint(new Vector3f(14, 0, 2));
		 path.addWayPoint(new Vector3f(20, 0, 6));
		 path.addWayPoint(new Vector3f(26, 0, 0));
		 path.addWayPoint(new Vector3f(20, 0, -6));
		 path.addWayPoint(new Vector3f(14, 0, 0));
		 path.addWayPoint(new Vector3f(20, 0, 6));
		 path.addWayPoint(new Vector3f(26, 0, 0));
		 path.addWayPoint(new Vector3f(20, 0, -6));
		 path.addWayPoint(new Vector3f(14, 0, 0));
		 
		 
		 
		 // loop 3
		 path.addWayPoint(new Vector3f(8, 0, 7.5f));
		 path.addWayPoint(new Vector3f(7, 0, 10.5f));
		 path.addWayPoint(new Vector3f(6, 0, 20));
		 path.addWayPoint(new Vector3f(0, 0, 26));
		 path.addWayPoint(new Vector3f(-6, 0, 20));
		 path.addWayPoint(new Vector3f(0, 0, 14));
		 path.addWayPoint(new Vector3f(6, 0, 20));
		 path.addWayPoint(new Vector3f(0, 0, 26));
		 path.addWayPoint(new Vector3f(-6, 0, 20));
		 path.addWayPoint(new Vector3f(0, 0, 14));
		 
		 
		 // begin elipse

		 path.addWayPoint(new Vector3f(16, 5, 20));
		 path.addWayPoint(new Vector3f(0, 0, 26));
		 path.addWayPoint(new Vector3f(-16, -10, 20));
		 path.addWayPoint(new Vector3f(0, 0, 14));
		 path.addWayPoint(new Vector3f(16, 20, 20));
		 path.addWayPoint(new Vector3f(0, 0, 26));
		 path.addWayPoint(new Vector3f(-10, -25, 10));
		 path.addWayPoint(new Vector3f(-10, 0, 0));
		 
		 // come at me bro!
		 path.addWayPoint(new Vector3f(-28.00242f, 48.005623f, -34.648228f));
		//path.addWayPoint(new Vector3f(0, 0 , -20));
		 
		 path.setCurveTension(0.80f);

		 
		 motionControl = new MotionTrack(bell,path);
	     motionControl.setDirectionType(MotionTrack.Direction.PathAndRotation);
	     motionControl.setRotation(new Quaternion().fromAngleNormalAxis(-FastMath.HALF_PI, Vector3f.UNIT_Y));
	     motionControl.setInitialDuration(10f);
	     motionControl.setSpeed(0.1f);
		 
		 
		 path.enableDebugShape(assetManager, rootNode);


		 positionCamera();
		
	        
	    }

	 
	 private void positionCamera(){
		//this.cam.setLocation(new Vector3f(-28.00242f, 48.005623f, -34.648228f));
		this.cam.setRotation(new Quaternion(0.3359635f, 0.34280345f, -0.13281013f, 0.8671653f));
	 }
	 
	 
	
	

  private AudioNode music;
  
  
  private Listener auxListener = new Listener(); 
  public File data1 = new File("/home/r/tmp/data1.wav");
  public File data2 = new File("/home/r/tmp/data2.wav");
  public File data3 = new File("/home/r/tmp/data3.wav");
   
  
  
  public void simpleInitApp() {
	this.setTimer(new IsoTimer(60));
	
	if (this.audioRenderer instanceof MultiListener){
		MultiListener rf = (MultiListener)this.audioRenderer;
		
		for (int n = 0; n < 0; n++){
			Listener zzz = new Listener();
			rf.addListener(zzz);
		}
		Listener listener3 = new Listener();
		rf.addListener(auxListener);
		rf.addListener(listener3);
		rf.registerSoundProcessor(new WaveFileWriter(data1));
		rf.registerSoundProcessor(auxListener, new WaveFileWriter(data2));
		rf.registerSoundProcessor(listener3, new WaveFileWriter(data3));
	}
    
    
    
        
    
    
    initAudio();
    initKeys();
	createScene();
	 

    motionControl.play();
	
	//this.audioRenderer.playSource(music);

	

  }

  /** We create two audio nodes. */ 
  private void initAudio() {
	//audioRenderer.setEnvironment(Environment.Cavern);
    
	music = new AudioNode(assetManager, "Sound/Environment/pure.wav", false);
	
	//music.setLooping(true);
    
   
	
    
    
    rootNode.attachChild(music);
    audioRenderer.playSource(music);
    
    music.setVolume(1f);
    music.setPositional(true);
    music.setMaxDistance(200.0f);
    music.setRefDistance(0.1f);
    music.setRolloffFactor(5f);
    audioRenderer.pauseSource(music);
    
    
    
  }

  /** Declaring the "Shoot" action, and
   *  mapping it to a trigger (mouse click). */
  

  /** Defining the "Shoot" action: Play a gun sound. */
  
  private void initKeys() {
	  inputManager.addMapping("Shoot", new MouseButtonTrigger(0));
	  inputManager.addListener(actionListener, "Shoot");
  }

  /** Defining the "Shoot" action: Play a gun sound. */
  private ActionListener actionListener = new ActionListener() {
	  @Override
	  public void onAction(String name, boolean keyPressed, float tpf) {
		  if (name.equals("Shoot") && !keyPressed) {
			  System.out.println("I'm playing! <3");
			  System.out.println(bell.getLocalTranslation().subtract(cam.getLocation()).length());	
			  
			  audioRenderer.playSource(music);
			  System.out.println(music.getRefDistance());
			 
		  }
	  }
  };

  /** Move the listener with the camera - for 3D audio. */
  @Override
  public void simpleUpdate(float tpf) {
	//Vector3f loc = cam.getLocation();
	//Quaternion rot = cam.getRotation();
    //listener.setLocation(loc);
    //listener.setRotation(rot);
    
    
    listener.setLocation(cam.getLocation());
    listener.setRotation(cam.getRotation());
    //auxListener.setLocation(loc);
    //auxListener.setRotation(rot);
    //if (music.getStatus() == AudioNode.Status.Stopped){

    	//music.playInstance();
    //}
    //audioRenderer.updateSourceParam(music, AudioParam.Direction);
    
    music.setLocalTranslation(bell.getLocalTranslation());
    
    //org.lwjgl.openal.AL10.alSourcef(1, org.lwjgl.openal.AL10.AL_MIN_GAIN, 0f);
    //org.lwjgl.openal.AL10.alSourcef(1, org.lwjgl.openal.AL10.AL_ROLLOFF_FACTOR, 5f);
    	
  }

}
